// Copyright AudioKit. All Rights Reserved. Revision History at http://github.com/AudioKit/AudioKit/

import AudioKit
import XCTest

class DistortionTests: XCTestCase {

    func testCubicTerm() {
        let engine = AudioEngine()
        let input = Oscillator()
        engine.output = Distortion(input, cubicTerm: 65)
        input.start()
        let audio = engine.startTest(totalDuration: 1.0)
        audio.append(engine.render(duration: 1.0))
        testMD5(audio)
    }

    func testDecay() {
        let engine = AudioEngine()
        let input = Oscillator()
        engine.output = Distortion(input, decay: 2)
        input.start()
        let audio = engine.startTest(totalDuration: 1.0)
        audio.append(engine.render(duration: 1.0))
        testMD5(audio)
    }

    func testDecimation() {
        let engine = AudioEngine()
        let input = Oscillator()
        engine.output = Distortion(input, decimation: 61)
        input.start()
        let audio = engine.startTest(totalDuration: 1.0)
        audio.append(engine.render(duration: 1.0))
        testMD5(audio)
    }

    func testDecimationMix() {
        let engine = AudioEngine()
        let input = Oscillator()
        engine.output = Distortion(input, decimationMix: 62)
        input.start()
        let audio = engine.startTest(totalDuration: 1.0)
        audio.append(engine.render(duration: 1.0))
        testMD5(audio)
    }

    func testDefault() {
        let engine = AudioEngine()
        let input = Oscillator()
        engine.output = Distortion(input)
        input.start()
        let audio = engine.startTest(totalDuration: 1.0)
        audio.append(engine.render(duration: 1.0))
        testMD5(audio)
    }

    func testDelay() {
        let engine = AudioEngine()
        let input = Oscillator()
        engine.output = Distortion(input, delay: 0.2)
        input.start()
        let audio = engine.startTest(totalDuration: 1.0)
        audio.append(engine.render(duration: 1.0))
        testMD5(audio)
    }

    func testDelayMix() {
        let engine = AudioEngine()
        let input = Oscillator()
        engine.output = Distortion(input, delayMix: 60)
        input.start()
        let audio = engine.startTest(totalDuration: 1.0)
        audio.append(engine.render(duration: 1.0))
        testMD5(audio)
    }

    func testFinalMix() {
        let engine = AudioEngine()
        let input = Oscillator()
        engine.output = Distortion(input, finalMix: 0.69)
        input.start()
        let audio = engine.startTest(totalDuration: 1.0)
        audio.append(engine.render(duration: 1.0))
        testMD5(audio)
    }

    func testLinearTerm() {
        let engine = AudioEngine()
        let input = Oscillator()
        engine.output = Distortion(input, linearTerm: 63)
        input.start()
        let audio = engine.startTest(totalDuration: 1.0)
        audio.append(engine.render(duration: 1.0))
        testMD5(audio)
    }

    func testParameters() {
        let engine = AudioEngine()
        let input = Oscillator()
        engine.output = Distortion(input,
                              delay: 0.2,
                              decay: 2,
                              delayMix: 60,
                              decimation: 61,
                              rounding: 50,
                              decimationMix: 62,
                              linearTerm: 63,
                              squaredTerm: 64,
                              cubicTerm: 65,
                              polynomialMix: 66,
                              ringModFreq1: 200,
                              ringModFreq2: 300,
                              ringModBalance: 67,
                              ringModMix: 68,
                              softClipGain: 0,
                              finalMix: 69)
        input.start()
        let audio = engine.startTest(totalDuration: 1.0)
        audio.append(engine.render(duration: 1.0))
        testMD5(audio)
    }

    func testPolynomialMix() {
        let engine = AudioEngine()
        let input = Oscillator()
        engine.output = Distortion(input, polynomialMix: 66)
        input.start()
        let audio = engine.startTest(totalDuration: 1.0)
        audio.append(engine.render(duration: 1.0))
        testMD5(audio)
    }

    func testRingModBalance() {
        let engine = AudioEngine()
        let input = Oscillator()
        engine.output = Distortion(input, ringModBalance: 67, ringModMix: 68)
        input.start()
        let audio = engine.startTest(totalDuration: 1.0)
        audio.append(engine.render(duration: 1.0))
        testMD5(audio)
    }

    func testRingModFreq1() {
        let engine = AudioEngine()
        let input = Oscillator()
        engine.output = Distortion(input, ringModFreq1: 200, ringModMix: 68)
        input.start()
        let audio = engine.startTest(totalDuration: 1.0)
        audio.append(engine.render(duration: 1.0))
        testMD5(audio)
    }

    func testRingModFreq2() {
        let engine = AudioEngine()
        let input = Oscillator()
        engine.output = Distortion(input, ringModFreq2: 300, ringModMix: 68)
        input.start()
        let audio = engine.startTest(totalDuration: 1.0)
        audio.append(engine.render(duration: 1.0))
        testMD5(audio)
    }

    func testRingModMix() {
        let engine = AudioEngine()
        let input = Oscillator()
        engine.output = Distortion(input, ringModMix: 68)
        input.start()
        let audio = engine.startTest(totalDuration: 1.0)
        audio.append(engine.render(duration: 1.0))
        testMD5(audio)
    }

    func testRounding() {
        let engine = AudioEngine()
        let input = Oscillator()
        engine.output = Distortion(input, rounding: 50)
        input.start()
        let audio = engine.startTest(totalDuration: 1.0)
        audio.append(engine.render(duration: 1.0))
        testMD5(audio)
    }

    func testSquaredTerm() {
        let engine = AudioEngine()
        let input = Oscillator()
        engine.output = Distortion(input, squaredTerm: 64)
        input.start()
        let audio = engine.startTest(totalDuration: 1.0)
        audio.append(engine.render(duration: 1.0))
        testMD5(audio)
    }

    func testSoftClipGain() {
        let engine = AudioEngine()
        let input = Oscillator()
        engine.output = Distortion(input, softClipGain: 0)
        input.start()
        let audio = engine.startTest(totalDuration: 1.0)
        audio.append(engine.render(duration: 1.0))
        testMD5(audio)
    }

}
