// Copyright AudioKit. All Rights Reserved. Revision History at http://github.com/AudioKit/AudioKit/

import AudioKit
import XCTest

class PitchShifterTests: XCTestCase {

    func testCrossfade() {
        let engine = AudioEngine()
        let input = Oscillator()
        engine.output = PitchShifter(input, shift: 7, crossfade: 1_024)
        input.start()
        let audio = engine.startTest(totalDuration: 1.0)
        audio.append(engine.render(duration: 1.0))
        testMD5(audio)
    }

    func testDefault() {
        let engine = AudioEngine()
        let input = Oscillator()
        engine.output = PitchShifter(input)
        input.start()
        let audio = engine.startTest(totalDuration: 1.0)
        audio.append(engine.render(duration: 1.0))
        testMD5(audio)
    }

    func testParameters() {
        let engine = AudioEngine()
        let input = Oscillator()
        engine.output = PitchShifter(input, shift: 7, windowSize: 2_048, crossfade: 1_024)
        input.start()
        let audio = engine.startTest(totalDuration: 1.0)
        audio.append(engine.render(duration: 1.0))
        testMD5(audio)
    }

    func testShift() {
        let engine = AudioEngine()
        let input = Oscillator()
        engine.output = PitchShifter(input, shift: 7)
        input.start()
        let audio = engine.startTest(totalDuration: 1.0)
        audio.append(engine.render(duration: 1.0))
        testMD5(audio)
    }

    func testWindowSize() {
        let engine = AudioEngine()
        let input = Oscillator()
        engine.output = PitchShifter(input, shift: 7, windowSize: 2_048)
        input.start()
        let audio = engine.startTest(totalDuration: 1.0)
        audio.append(engine.render(duration: 1.0))
        testMD5(audio)
    }

}
